"""
meshes_container
================
Autogenerated DPF operator classes.
"""

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class meshes_container(Operator):
    """Incrementaly merge the input.

    Parameters
    ----------
    input : MeshesContainer

    Returns
    -------
    incremented_result :

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.meshes_container()

    >>> # Make input connections
    >>> my_input = dpf.MeshesContainer()
    >>> op.inputs.input.connect(my_input)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.meshes_container(
    ...     input=my_input,
    ... )

    >>> # Get output data
    >>> result_incremented_result = op.outputs.incremented_result()
    """

    def __init__(self, input=None, config=None, server=None):
        super().__init__(
            name="incremental::merge::meshes_container", config=config, server=server
        )
        self._inputs = InputsMeshesContainer(self)
        self._outputs = OutputsMeshesContainer(self)
        if input is not None:
            self.inputs.input.connect(input)

    @staticmethod
    def _spec():
        description = """Incrementaly merge the input."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="input",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="incremented_result",
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="incremental::merge::meshes_container", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshesContainer
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshesContainer
        """
        return super().outputs


class InputsMeshesContainer(_Inputs):
    """Intermediate class used to connect user inputs to
    meshes_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.meshes_container()
    >>> my_input = dpf.MeshesContainer()
    >>> op.inputs.input.connect(my_input)
    """

    def __init__(self, op: Operator):
        super().__init__(meshes_container._spec().inputs, op)
        self._input = Input(meshes_container._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._input)

    @property
    def input(self):
        """Allows to connect input input to the operator.

        Parameters
        ----------
        my_input : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.meshes_container()
        >>> op.inputs.input.connect(my_input)
        >>> # or
        >>> op.inputs.input(my_input)
        """
        return self._input


class OutputsMeshesContainer(_Outputs):
    """Intermediate class used to get outputs from
    meshes_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.meshes_container()
    >>> # Connect inputs : op.inputs. ...
    >>> result_incremented_result = op.outputs.incremented_result()
    """

    def __init__(self, op: Operator):
        super().__init__(meshes_container._spec().outputs, op)
        self._incremented_result = Output(meshes_container._spec().output_pin(0), 0, op)
        self._outputs.append(self._incremented_result)

    @property
    def incremented_result(self):
        """Allows to get incremented_result output of the operator

        Returns
        ----------
        my_incremented_result :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.meshes_container()
        >>> # Connect inputs : op.inputs. ...
        >>> result_incremented_result = op.outputs.incremented_result()
        """  # noqa: E501
        return self._incremented_result
